<?php // Include only
if (!defined ('BOOT')) exit ('No bootstrap.  Exiting.');

/*
Wax Poteto database layer Copyright 2010 Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 5.8.2 - Last modified 12/26/2010

NOTE: Should be identical to Wacintaki version (1.5.5)

Fixes MySQL 5 password support, improves error reporting.  First step towards a real database layer.

Notes:
	PHP < 5.3 does not support namespaces, so for compatibility, "global" is used a lot, here.  I've had some trouble using "$GLOBALS['si']" with PHP4, so "global $si" is used for each function.

	OO style is not used for MySQLi, because MySQL doesn't support it, and all DB calls in Wacintaki are procedural style.

	MySQLi does not work on PHP4, but for some strange reason, some PHP4 people have been telling me that the board will detect MySQLi and try to use it.  It must be explicitly disabled for PHP4.  No, I don't get it, either.

Todo:
	history/error caches more than one message

*/


// Init
$dbconn    = NULL;
$dbcharset = NULL;
// $wactest = (Debug var set in hacks.php file)

// Config must be public for backwards compatibility
if (!defined ('PARSE_DBCONN_MANUALLY') || !PARSE_DBCONN_MANUALLY) {
	require ('dbconn.php');
}

// Remap prefixes
$db_p   = $OekakiPoteto_Prefix;
$db_mem = $OekakiPoteto_MemberPrefix;
$db_op  = $db_p;   // Deprecated
$db_pre = $db_p;   // Deprecated
$db_mem = $db_mem; // Deprecated

// Do not use outside of db_layer.php!
$db_error = '';
$db_history = '';


/*
	Use MySQL or MySQLi?
	On some PHP4 servers, extension and connect function exists, but queries still fail.
	This may be because of 'sql.safe_mode' or attempting to use a PHP5 config file with PHP4.
	We'll use MySQLi with PHP5 only.
*/
$si = FALSE;
if (extension_loaded ('mysqli') && function_exists ('mysqli_connect')) {
	// Note: boot.php defines 'PHP_VERSION_ID' for PHP < 5.2.7
	if (defined ('PHP_MAJOR_VERSION') || PHP_VERSION_ID > 49999) {
		$si = TRUE;
	}
}



function db_backtrace() {
	// Gets backtrace to figure out line number for bad db_*() call

	next (debug_backtrace());
	next (debug_backtrace());
	$debug = next(debug_backtrace());
	end (debug_backtrace());

	$info = pathinfo ($debug['file']);

	return ("[{$info['basename']} > db_result() > {$debug['line']}]");
}

function db_trigger() {
	global $si, $dbconn, $wactest;

	$marker = db_backtrace();

	trigger_error ($marker, E_USER_WARNING);
	if ($wactest) {
		echo ('<b>DB: '.db_error().'</b><br />');
	}

	return;
}

function db_query($q) {
	global $si, $dbconn, $wactest;

	db_push_history($q);

	// Note: Queries do not trigger errors.  Do not mask with "@".
	if ($si) {
		$result = mysqli_query ($dbconn, $q);
	} else {
		$result = mysql_query ($q, $dbconn);
	}

	if ($result === FALSE) {
		$marker = db_backtrace();
		db_push_error($marker.' '.db_error ($dbconn));

		if ($wactest) {
			echo ('<b>'.$marker.'</b> DB: '.db_error().' <b>(Q: '.htmlspecialchars (db_history()).')</b><br />');
		}
	}

	return ($result);
}

function db_result ($result, $offset = 0) {
	global $si, $dbconn, $wactest;

	$data = NULL;
	$array = NULL;

	if ($si) {
		if (mysqli_data_seek ($result, $offset)) {
			$array = mysqli_fetch_array ($result, MYSQLI_NUM);
			$data = $array[0];
		}
	} else {
		$data = @mysql_result ($result, $offset);
	}

	if ($data === FALSE) {
		$marker = db_backtrace();
		db_push_error(db_error());

		if ($wactest) {
			echo ('<b>'.$marker.'</b> DB: '.db_error().' <b>(Q: '.htmlspecialchars (db_history()).')</b><br />');
		} else {
			db_trigger();
		}
	}

	return ($data);
}

function db_fetch_array($result, $type = NULL) {
	global $si, $dbconn, $wactest;

	if (defined ('MYSQLI_ASSOC') && empty ($type)) {
		$type = MYSQLI_ASSOC;
	}

	/*
		Notes:

		- Unlike the native mysql functions, this function always
		returns associative arrays by default.

		- Due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for "FALSE" here.  Do not mask errors with "@".
	*/
	if ($si) {
		return (mysqli_fetch_array ($result, $type));
	} else {
		return (mysql_fetch_array ($result));
	}
}

function db_fetch_row($result) {
	global $si, $dbconn, $wactest;

	/*
		Note: due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for "FALSE" here.  Do not mask errors with "@".
	*/
	if ($si) {
		return (mysqli_fetch_row ($result));
	} else {
		return (mysql_fetch_row ($result));
	}
}

function db_num_rows($result) {
	global $si, $dbconn;

	if ($si) {
		return (mysqli_num_rows ($result));
	} else {
		return (mysql_num_rows ($result));
	}
}

function db_affected_rows() {
	global $si, $dbconn;

	if ($si) {
		return (mysqli_affected_rows($dbconn));
	} else {
		// WARNING: Make sure $dbconn is global, even though it is not passed.
		// PHP does some magic here, and passing $dbconn explicitly causes "trouble".
		return (mysql_affected_rows());
	}
}

function db_free_result($result) {
	global $si, $dbconn;

	if ($si) {
		return (@mysqli_free_result($dbconn, $result));
	} else {
		return (@mysql_free_result($result));
	}
}

function db_error($offset = 0) {
	global $si, $dbconn, $db_error;

	if ($dbconn) {
		if ($si) {
			return (@mysqli_error($dbconn));
		} else {
			return (mysql_error());
		}
	} else {
		return ('No connection');
	}
}

//function db_num_error() {
//	global $db_error;
//	return (count ($db_error));
//}

function db_push_error($in) {
	global $si, $db_error;

	if (!empty ($in)) {
		$db_error = $in;
	}

	return;
}

function db_history($offset = 0) {
	global $si, $db_history;

	return ($db_history);
}

//function db_num_history() {
//	global $db_history;
//	return (count ($db_history));
//}

function db_push_history($in) {
	global $si, $db_history;

	if (!empty ($in)) {
		$db_history = $in;
	}

	return;
}

function db_escape($in) {
	global $si, $dbconn;

	if (is_array ($in)) {
		// Let godmode sort 'em out
		return '';
	} elseif ($si) {
		return (mysqli_real_escape_string ($in, $dbconn));
	} else {
		if (function_exists ('mysql_real_escape_string')) {
			return (mysql_real_escape_string ($in, $dbconn));
		} else {
			// Old PHP (no charset support)
			return (mysql_escape_string ($in, $dbconn));
		}
	}
}

function db_set_charset($charset = 'utf8') {
	global $si, $dbconn;

	// Requires PHP5 and MySQL 5.0.8
	$my_version = explode ('.', PHP_VERSION);

	$done = FALSE;
	if ($my_version[0] > 4) {
		if ($si) {
			if (function_exists('mysqli_set_charset')) {
				$done = mysqli_set_charset($dbconn, $charset);
			}
		} else {
			if (function_exists('mysql_set_charset')) {
				$done = @mysql_set_charset($charset);
			}
		}
	}

	return ($done);
}

function db_get_charset() {
	global $si, $dbconn;

	if ($si) {
		return( mysqli_character_set_name($dbconn) );
	} else {
		return( mysql_client_encoding($dbconn) );
	}
}

function db_open() {
	global $si, $dbhost, $dbuser, $dbpass, $db_error, $dbname;

	$db_type = 'MySQL';

	if ($si) {
		$db_type = 'MySQLi';

		$GLOBALS['dbconn'] = @mysqli_connect ($dbhost, $dbuser, $dbpass);
		if (!$GLOBALS['dbconn']) {
			exit ('Could not connect to the SQL database (API: MySQLi)<br />DB: '.mysqli_connect_error() );
		}
	} else {
		$GLOBALS['dbconn'] = mysql_connect ($dbhost, $dbuser, $dbpass);
		if (!$GLOBALS['dbconn']) {
			exit ('Could not connect to the SQL database (API: MySQL)<br />DB: '.mysql_error() );
		}
	}

	$mySQLDB = db_select($GLOBALS['dbconn'], $dbname);
	if (!$mySQLDB) {
		db_close();
		exit ('Database name &ldquo;'.htmlspecialchars ($dbname).'&rdquo; is invalid (API: '.$db_type.')');
	}

	// UTF-8 is not yet active!  Do NOT enable!
	// db_set_charset('utf8');
	$GLOBALS['dbcharset'] = db_get_charset();

	return ($GLOBALS['dbconn']);
}

function db_select($conn, $dbname) {
	global $si;

	if ($si) {
		return (@mysqli_select_db ($conn, $dbname));
	} else {
		return (@mysql_select_db ($dbname));
	}
}

function db_close($unused = 0) {
	// Don't global $dbconn here.  PHP may throw an undeclared variable error.
	global $si;

	if ($si) {
		if (isset($GLOBALS['dbconn']) && !empty ($GLOBALS['dbconn'])) {
			return (@mysqli_close ($GLOBALS['dbconn']));
		} else {
			return (@mysqli_close());
		}
	} else {
		if (isset($GLOBALS['dbconn']) && !empty ($GLOBALS['dbconn'])) {
			return (@mysql_close ($GLOBALS['dbconn']));
		} else {
			return (@mysql_close());
		}
	}

	$GLOBALS['dbconn'] = NULL;
}

?>